﻿/********************************************************************************/
/* 	character.d																	*/
/*------------------------------------------------------------------------------*/
/*	製作		D.K（あるふぁ～秘密基地）										*/
/*	製作開始	2008/05/22														*/
/*	MAIL		dk@red.interq.or.jp												*/
/*	URL			http://maglog.jp/alpha-secret-base/								*/
/*																				*/
/*	このソースは「やわらかライセンス」の元で配布されています。					*/
/*-更新履歴---------------------------------------------------------------------*/
/*	2008/05/22	初版															*/
/*-その他-----------------------------------------------------------------------*/
/*	なし																		*/
/********************************************************************************/

private import std.ctype;
private import std.file;
private import std.string;
private import std.date;
private import std.conv;

private import gamecore;
private import hell2;
private import boot;
private import actor;
private import vector;
private import keypad;
private import map;

//********************************************************************************
// キャラクタ
//********************************************************************************

class Character : Actor
{
	enum CHRTYPE{
		NONE = 0,
		
		PLAYER_START,
		PLAYER1 = PLAYER_START,
		PLAYER2,
		PLAYER_END,

		EVENT_START = 16,
		SCROLL_FREE = EVENT_START,
		SCROLL_RIGHT,
		SCROLL_LEFT,
		SCROLL_UP,
		SCROLL_DOWN,
		SCROLL_WAIT,
		BG_SET,
		EVENT_END,

		ENEMY_START = 32,
		ENEMY1 = ENEMY_START,
		ENEMY_END,

		ITEM_START = 48,
		ITEM1 = ITEM_START,
		ITEM2,
		ITEM_END,

		PLAYER_SHOT_START,
		PLAYER_SHOT1 = PLAYER_SHOT_START,
		PLAYER_SHOT_END,

		ENEMY_SHOT_START,
		ENEMY_SHOT,
		ENEMY_SHOT_END,

		EFFECT_START,
		EFFECT1 = EFFECT_START,
		EFFECT2,
		EFFECT3,
		EFFECT4,
		EFFECT5,
		EFFECT_END,

		MOB_START,
		MOB1 = MOB_START,
		MOB_END,

		MAX,
	};

	enum MOTION{
		NONE = 0,
		NEUTRAL,
		WALKING,
		JUMP_UP,
		JUMP_DOWN,
		DAMAGE,
		DOWN,
		CLEAR,
		ATTACK,
		JUMP_ATTACK,
		
		YOBI1,
		YOBI2,
		YOBI3,
		YOBI4,
		YOBI5,
		YOBI6,
		YOBI7,
		
		MAX,
	};
	// 方向
	enum DIRECTION{
		NONE = 0,
		RIGHT,
		LEFT,
		MAX,
	}
	
	// 場所
	enum PLACE{
		NONE = 0,
		GROUND,
		AIR,
		MAX,
	}

	GameMain gamemain;
	CHRTYPE type;
	Keypad keypad;

	//敵用仮想キーコード
	int Vartalkey_Up;
	int Vartalkey_Down;
	int Vartalkey_Left;
	int Vartalkey_Right;
	int Vartalkey_Botton1;
	int Vartalkey_Botton2;
	int Vartalkey_Botton3;
	//敵用行動状態
	int enemy_seqense;
	Vec3 player_position;			//キャラクターの位置
	//キー入力判定の有無
	int is_inputkey;
	

	MOTION motion;				//モーション
	int anime_pattern_count;	//アニメーションパターンカウンタ
	int anime_pattern_max;		//アニメーションパターン最大数
	float anime_frame_count;	//アニメーション更新フレームカウンタ
	int anime_update_frame;		//アニメーション更新フレーム
	int anime_start_x;			//アニメーション画像の開始位置x
	int anime_start_y;			//アニメーション画像の開始位置y
	int anime_width;			//アニメーション画像の幅
	int anime_height;			//アニメーション画像の高さ
	int anime_endpattern_chg;	//アニメーション終了後のパターンを変えるか 0：指定パターンに戻る、1：別のモーションに変える
	int anime_endpattern;		//アニメーション終了後のパターン
	int anime_endpattern_motion;//アニメーション終了後のモーション
	int anime_reverse;			//アニメーション向きによる反転機能有無
	int char_width;				//キャラクターの当たり判定幅、中心から
	int char_height;			//キャラクターの当たり判定高さ、中心から
	static const ANIMETABLE_MAX = 20;
	int motion_date[MOTION.MAX][ANIMETABLE_MAX];	//ファイルから読み込んだモーションテーブル[モーション番号][情報]
	MOTION old_motion;						//前フレームのモーション

	DIRECTION direction;		// 向き
	PLACE place;				//場所
	float MOVE_SPEED = 2;		//移動速度

	int jump_fream_count;			//ジャンプフレームカウンタ
	int JUMP_UPDATE_FREAM = 10;		//ジャンプフレーム更新時間
	int JUMP_COUNT_MAX = 10;		//ジャンプ力度最大
	int JUMP_COUNT_MIN = -4;		//ジャンプ力度最小
	int JUMP_COUNT_MOVE = 2;		//ジャンプ力の変化量
	int jump_count;					//ジャンプカウンタ
	int JUMP_SCALE = 20;			//ジャンプ力スケール

	float disable_input_count;	//入力不可フレーム数カウンタ
	float no_hit_count = 0;		//無敵時間
	float disp_rotate;			//表示角度
	int life;					//体力

	int is_draw_one;			//画面内に１度でも表示を行ったか
	int is_draw;				//表示行うか

	int debug_disp[10];			//デバッグ用ワーク

	//動作バッファ
	int move_buff[20];
	int param;					//パラメータ

	float map_rotate;			//地面の角度
	float map_size;				//地面のサイズ
	float map_speed;			//ゲームスピード

	int char_width_start;		//モーション開始時の当たり判定

	//-----------------------------------------------------------------------------
	//	コンストラクタ
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	this()
	{
		player_position = new Vec3();
		position = new Vec3();
		type = CHRTYPE.NONE;
	}

	//-----------------------------------------------------------------------------
	//	初期化
	//-----------------------------------------------------------------------------
	//	引	数: GameMain：シーンクラス
	//			Vec3：位置情報
	//			CHRTYPE：キャラクターのタイプ
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void init( GameMain gamemain , Vec3 pos , CHRTYPE chartype )
	{
		int i;
		this.gamemain = gamemain;
		this.position = pos;
		this.type = chartype;
		this.exist = true;

		this.motion = MOTION.NEUTRAL;
		this.keypad = null;
		this.place = PLACE.GROUND;
		this.disable_input_count = 0;
		this.disp_rotate = 0;			//表示角度
		this.jump_fream_count = 0;
		this.jump_count = 0;
		this.is_draw_one = 0;			//画面内に１度でも表示を行ったか
		this.is_draw = 1;				//表示行うか
		this.life = 1;
		this.param = 0;
		this.is_inputkey = 0;
		this.map_rotate = 0;
		this.map_size = 0;
		this.map_speed = 1.0f;

		//敵用仮想キーコード
		this.Vartalkey_Up = 0;
		this.Vartalkey_Down = 0;
		this.Vartalkey_Left = 0;
		this.Vartalkey_Right = 0;
		this.Vartalkey_Botton1 = 0;
		this.Vartalkey_Botton2 = 0;
		this.Vartalkey_Botton3 = 0;
		//敵用行動状態
		this.enemy_seqense = 0;
		
		//動作バッファのクリア
		for ( i = 0; i < 20; i++ )
		{
			this.move_buff[i] = 0;
		}
		if ( 
		     ( type == CHRTYPE.PLAYER1 ) 
		  || ( type == CHRTYPE.PLAYER2 ) 
		  || ( type == CHRTYPE.ENEMY1 ) 
		   )
		{
//			this.keypad = new Keypad();
			this.keypad = gamemain.keypad;
			motion_load( );
			motiondata_set( );
		}
		if ( 
		     ( type == CHRTYPE.PLAYER1 ) 
		  || ( type == CHRTYPE.PLAYER2 ) 
		   )
		{
			//キー入力あり
			is_inputkey = 1;
		}
	
		switch( chartype )
		{
		case CHRTYPE.PLAYER1:
		case CHRTYPE.PLAYER2:
			move_buff[0] = 0;		//ドアに重なった
			move_buff[1] = 0;		//ワープが可能
			move_buff[2] = -1;		//アイテム取得情報
			move_buff[3] = -1;		//アイテム取得情報
			move_buff[4] = -1;		//アイテム取得情報
			move_buff[5] = 0;		//走り続けたボーナスのカウント
			move_buff[6] = 0;		//今回のフレームは移動を行ったか？
			break;
		case CHRTYPE.ENEMY1:
			MOVE_SPEED = 1.8;//移動速度
			move_buff[0] = 1;		//地中から上る
			move_buff[1] = 0;		//方向転換を行ったか
			move_buff[2] = 0;		//パワーアップ中
			move_buff[3] = 0;		//Uターン表示カウンタ
			move_buff[4] = 0;		//プレイヤーにあたった
			break;
		case CHRTYPE.ITEM1:
			char_width = 32;	//当たり判定の幅
			char_height = 32;	//当たり判定の高さ
			move_buff[0] = 0;	//取得された
			break;
		case CHRTYPE.ITEM2:
			char_width = 64;	//当たり判定の幅
			char_height = 32;	//当たり判定の高さ
			move_buff[0] = 0;	//プレイヤーがドアに重なっている
			break;
		case CHRTYPE.EFFECT2:
			//移動方向と速度をランダムできめる
			move_buff[5] = Hell_randInt( 5, 10 );	//速度
			move_buff[6] = Hell_randInt( 0, 359 );	//角度
			break;
		case CHRTYPE.EFFECT3:
			move_buff[0] = 0;	//移動カウンタ
			move_buff[1] = 10;	//移動速度
			move_buff[2] = 0;	//メッセージの種類
			break;
		case CHRTYPE.EFFECT4:
			move_buff[0] = 0;	//死亡までのカウンタ
			break;
		case CHRTYPE.MOB1:
			move_buff[0] = 1;	//アニメーションカウンタ
			move_buff[1] = 100;	//うねうね
			move_buff[2] = 100;	//うねうね
			move_buff[3] = 0;	//うねうね
			move_buff[4] = gamemain.time_extend_count;	//出現時のエクステンド回数
			move_buff[5] = 1200;	                    /*【変更01_001】にぎやかし(建物の)明暗制御カウンタを追加。初期値1200（RGB各120相当）*/
			break;
		default:
			break;
		}

	}
	//-----------------------------------------------------------------------------
	//	モーション情報の読み込み
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void motion_load( )
	{
		int i = 0;
		int j = 0;
		
		switch( type )
		{
		case CHRTYPE.PLAYER1:
		case CHRTYPE.PLAYER2:
			auto file = cast( char[] )std.file.read( "data\\char\\player.csv" );
			string[] buf1 = split( file, "\n" );
			
			for ( i = 0; buf1.length > i; i++ )
			{
				if ( i == 0 )			//先頭行
				{
					continue;
				}
				if ( i >= MOTION.MAX )	//バッファを越えている場合
				{
					continue;
				}
				string[] buf2 = split( buf1[i], "," );
				for ( j = 0; buf2.length > j; j++ )
				{
					motion_date[i][j] = 0;		//初期化
					switch( j )
					{
					case 0:
						//モーション番号固定
						break;
					case 1:
						//アニメーション名
						break;
					case 2:
					case 3:
					case 4:
					case 5:
					case 6:
					case 7:
					case 8:
					case 9:
					case 10:
					case 11:
					case 12:
					case 13:
						motion_date[i][j] = cast( int )atoi( cast( char[] )buf2[j] );		//アニメーションパターン最大数
						break;
					default:
						break;
					}
				}
			}
			break;
		case CHRTYPE.ENEMY1:
			auto file = cast( char[] )std.file.read( "data\\char\\enemy1.csv" );
			string[] buf1 = split( file, "\n" );
			
			for ( i = 0; buf1.length > i; i++ )
			{
				if ( i == 0 )			//先頭行
				{
					continue;
				}
				if ( i >= MOTION.MAX )	//バッファを越えている場合
				{
					continue;
				}
				string[] buf2 = split( buf1[i], "," );
				for ( j = 0; buf2.length > j; j++ )
				{
					motion_date[i][j] = 0;		//初期化
					switch( j )
					{
					case 0:
						//モーション番号固定
						break;
					case 1:
						//アニメーション名
						break;
					case 2:
					case 3:
					case 4:
					case 5:
					case 6:
					case 7:
					case 8:
					case 9:
					case 10:
					case 11:
					case 12:
					case 13:
						motion_date[i][j] = cast( int )atoi( cast( char[] )buf2[j] );		//アニメーションパターン最大数
						break;
					default:
						break;
					}
				}
			}
			break;
		default:
			break;
		}
	}
	//-----------------------------------------------------------------------------
	//	モーション情報の設定
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void motiondata_set( )
	{
		int j = 0;
		for ( j = 0; ANIMETABLE_MAX > j; j++ )
		{
			switch( j )
			{
			case 2:
				//パターン最大数
				anime_pattern_max = motion_date[motion][j];		//アニメーションパターン最大数
				break;
			case 3:
				//更新フレーム
				anime_update_frame = motion_date[motion][j];	//アニメーション更新フレーム
				break;
			case 4:
				//画像開始位置Ｘ
				anime_start_x = motion_date[motion][j];			//アニメーション画像の開始位置x
				break;
			case 5:
				//画像開始位置Ｙ
				anime_start_y = motion_date[motion][j];			//アニメーション画像の開始位置y
				break;
			case 6:
				//画像の幅
				anime_width = motion_date[motion][j];			//アニメーション画像の幅
				break;
			case 7:
				//画像の高さ
				anime_height = motion_date[motion][j];			//アニメーション画像の高さ
				break;
			case 8:
				//終了後パターン変更指定
				anime_endpattern_chg = motion_date[motion][j];	//アニメーション終了後のパターンを変えるか 0：指定パターンに戻る、1：別のモーションに変える
				break;
			case 9:
				//終了後のパターン
				anime_endpattern = motion_date[motion][j];		//アニメーション終了後のパターン
				break;
			case 10:
				//終了後のモーション
				anime_endpattern_motion = motion_date[motion][j];//アニメーション終了後のモーション
				break;
			case 11:
				//向きによる反転機能有無
				anime_reverse = motion_date[motion][j];			//アニメーション向きによる反転機能有無
				break;
			case 12:
				//当たり判定幅
				char_width = motion_date[motion][j];			//当たり判定幅
				break;
			case 13:
				//当たり判定高さ
				char_height = motion_date[motion][j];			//当たり判定高さ
				break;
			default:
				break;
			}
		}
	}
	
	//-----------------------------------------------------------------------------
	//	パラメータの設定
	//-----------------------------------------------------------------------------
	//	引	数: int add_param：パラメータ
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void set_param( int add_param )
	{
		param = add_param;					//パラメータ
	}
	
	//-----------------------------------------------------------------------------
	//	動作のバッファの設定
	//-----------------------------------------------------------------------------
	//	引	数: int buffindex：バッファインデックス
	//			int add_param：パラメータ
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void set_buffer( int buffindex, int param )
	{
		move_buff[buffindex] = param;					//パラメータ
	}
	
	//-----------------------------------------------------------------------------
	//	場所の取得
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: Vec3：位置情報
	//-----------------------------------------------------------------------------
	Vec3 getPosition( )
	{
		return position;
	}
	//-----------------------------------------------------------------------------
	//	当たり判定の取得
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: Vec3：位置情報
	//-----------------------------------------------------------------------------
	int get_width( )
	{
		return char_width;
	}
	//-----------------------------------------------------------------------------
	//	当たり判定の取得
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: Vec3：位置情報
	//-----------------------------------------------------------------------------
	int get_height( )
	{
		return char_height;
	}


	//-----------------------------------------------------------------------------
	//	プレイヤー座標の設定
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: Vec3：位置情報
	//-----------------------------------------------------------------------------
	void setPlayerPosition( Vec3 pos )
	{
		player_position = pos;
	}
	
	//-----------------------------------------------------------------------------
	//	マップの情報を設定
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: Vec3：位置情報
	//-----------------------------------------------------------------------------
	void setMapRotate( float rotate )
	{
		map_rotate = ( rotate % 360 );
	}
	void setMapSize( float size )
	{
		map_size = size;
	}
	void setGameSpeed( float speed )
	{
		map_speed = speed;
	}

	//-----------------------------------------------------------------------------
	//	移動
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void move( )
	{
		Vec3 old_pos = new Vec3( position );			//前フレームのキャラタ座標
		int old_is_draw_one = is_draw_one;				//前フレームで表示を行ったか

		//アニメーションの更新
		anime_frame_count = anime_frame_count + map_speed;											//フレームのカウント
		if ( anime_frame_count >= anime_update_frame )					//更新フレームを超えた
		{
			anime_frame_count = 0;										//フレームカウントのクリア
			anime_pattern_count++;										//パターンの更新
			if ( anime_pattern_count >= anime_pattern_max )				//最終パターンが終了した
			{
				switch ( anime_endpattern_chg )							//終了後の動作を決める
				{
				case 0:
					anime_pattern_count = anime_endpattern;				//パターンを戻す
					break;
				case 1:
					motion = cast( MOTION )anime_endpattern_motion;		//モーションを帰る
					break;
				default:
					break;
				}
			}
		}
		//入力不可時間カウント
		disable_input_count = disable_input_count - map_speed;
		if ( disable_input_count < 0 )
		{
			disable_input_count = 0;
		}
		//無敵時間のカウント
		no_hit_count = no_hit_count - map_speed;
		if ( no_hit_count < 0 )
		{
			no_hit_count = 0;
		}
		
		switch( type )
		{
		case CHRTYPE.PLAYER1:
		case CHRTYPE.PLAYER2:
			if ( disable_input_count == 0 )
			{
				//プレイヤー
				// 移動方向決定
				if(keypad.isPressDown())
				{
					//下
//					position.y = position.y + 10;
				}
				else if(keypad.isPressUp())
				{
					//上
//					position.y = position.y - 10;
				}
				
				if(keypad.isPressLeft())
				{
					move_buff[5] = move_buff[5] + 1;		//走り続けたボーナスのカウント
					//左
					direction = DIRECTION.LEFT;
					switch( motion )
					{
					case MOTION.NEUTRAL:	//入力なし
						motion = MOTION.WALKING;
					case MOTION.WALKING:	//歩き
						move_buff[6] = 1;	//今回のフレームは移動を行ったか？
						position.x = position.x - ( MOVE_SPEED * map_speed );
						break;
					case MOTION.JUMP_UP:		//ジャンプ上昇
					case MOTION.JUMP_DOWN:		//ジャンプ下降
						move_buff[6] = 1;	//今回のフレームは移動を行ったか？
						position.x = position.x - ( MOVE_SPEED * map_speed );
						break;
					case MOTION.DAMAGE:			//ダメージ
					case MOTION.DOWN:			//ゲームオーバー
					case MOTION.CLEAR:			//クリア
					case MOTION.ATTACK:			//地上攻撃
					case MOTION.JUMP_ATTACK:	//空中攻撃
					default:
						break;
					}
				}
				else if(keypad.isPressRight())
				{
					move_buff[5] = move_buff[5] + 1;		//走り続けたボーナスのカウント
					//右
					direction = DIRECTION.RIGHT;
					switch( motion )
					{
					case MOTION.NEUTRAL:	//入力なし
						motion = MOTION.WALKING;
					case MOTION.WALKING:	//歩き
						move_buff[6] = 1;	//今回のフレームは移動を行ったか？
						position.x = position.x + ( MOVE_SPEED * map_speed );
						break;
					case MOTION.JUMP_UP:		//ジャンプ上昇
					case MOTION.JUMP_DOWN:		//ジャンプ下降
						move_buff[6] = 1;	//今回のフレームは移動を行ったか？
						position.x = position.x + ( MOVE_SPEED * map_speed );
						break;
					case MOTION.DAMAGE:			//ダメージ
					case MOTION.DOWN:			//ゲームオーバー
					case MOTION.CLEAR:			//クリア
					case MOTION.ATTACK:			//地上攻撃
					case MOTION.JUMP_ATTACK:	//空中攻撃
					default:
						break;
					}
				}
				else 
				{
					if ( place == PLACE.GROUND )
					{
						move_buff[5] = 0;		//走り続けたボーナスのカウント
						motion = MOTION.NEUTRAL;	//操作していないときはニュートラルにする
					}
				}
				
				if(keypad.isPushEnter())
				{
					//１ボタン
					switch( motion )
					{
					case MOTION.NEUTRAL:	//入力なし
					case MOTION.WALKING:	//歩き
						if ( move_buff[1] == 1 )	//ワープ可能
						{
							Hell_playWAV( "door", 0, -1 );
							//親に演出要求
							//ワープ演出の開始
							no_hit_count = map_speed;  /*【変更03_0001】ドア使用と同時にミスとなってドアの音だけ再生されるのでこのときにドア使用を優先するために無敵時間を付加*/
							if ( direction == DIRECTION.LEFT )
							{
								gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_PLAYER_WARP_L, 0, 0, 0, 0, 0 );
							}
							else 
							{
								gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_PLAYER_WARP_R, 0, 0, 0, 0, 0 );
							}
						}
						break;
					case MOTION.JUMP_UP:		//ジャンプ上昇
					case MOTION.JUMP_DOWN:		//ジャンプ下降
					case MOTION.DAMAGE:			//ダメージ
					case MOTION.DOWN:			//ゲームオーバー
					case MOTION.CLEAR:			//クリア
					case MOTION.ATTACK:			//地上攻撃
					case MOTION.JUMP_ATTACK:	//空中攻撃
					default:
						break;
					}
				}
/*
				if(keypad.isPushCancel())
				{
					//２ボタン
					if ( place == PLACE.GROUND )
					{
						jump_start( 5 );			//ジャンプ力の設定
						//地上にいる
						switch( motion )
						{
						case MOTION.NEUTRAL:	//入力なし
						case MOTION.WALKING:	//歩き
							motion = MOTION.JUMP_UP;
							break;
						case MOTION.JUMP_UP:		//ジャンプ上昇
						case MOTION.JUMP_DOWN:		//ジャンプ下降
						case MOTION.DAMAGE:			//ダメージ
						case MOTION.DOWN:			//ゲームオーバー
						case MOTION.CLEAR:			//クリア
						case MOTION.ATTACK:			//地上攻撃
						case MOTION.JUMP_ATTACK:	//空中攻撃
						default:
							break;
						}
					}
				}
*/
			}
			else 
			{
				//硬直中
				move_buff[5] = 0;		//走り続けたボーナスのカウント
			}
			
//			if ( move_buff[5] >= tun_time ) 		//走り続けたボーナスのカウント
//			{
//				move_buff[5] = 0;		//走り続けたボーナスのカウント
//				//走り続けたボーナス
//				gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_SCOER_UP_NONSTOP, 0, 0, 0, move_buff[5], 0 );
//				Hell_playWAV( "get", 0, -1 );
//			}
			
			//アイテムの取得を判定しパワーアップをする
			if ( 
			     ( move_buff[2] != -1 )
			  && ( move_buff[3] != -1 )
			  && ( move_buff[4] != -1 )
			   )
			{
				//3つ取得した
				if ( 
				     ( move_buff[2] != move_buff[3] )
				  && ( move_buff[3] != move_buff[4] )
				  && ( move_buff[4] != move_buff[2] )
				   )
				{
					//３つとも違う
					//パワーアップ要求を行う
					move_buff[2] = -1;
					move_buff[3] = -1;
					move_buff[4] = -1;
					gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_POWER_UP, 0, 0, 0, 0, 0 );
					
				}
				else if ( 
				     ( move_buff[2] == move_buff[3] )
				  && ( move_buff[3] == move_buff[4] )
				  && ( move_buff[4] == move_buff[2] )
				   )
				{
					//３つとも違う
					//パワーアップ要求を行う
					move_buff[2] = -1;
					move_buff[3] = -1;
					move_buff[4] = -1;
					gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_SCOER_UP_ITEM3GET, 0, 0, 0, 0, 0 );
				}
			}
			
			if ( position.x < 0 )
			{
				position.x = position.x + gamemain.getMapWidth( );
			}
			if ( position.x >= gamemain.getMapWidth( ) )
			{
				position.x = position.x - gamemain.getMapWidth( );
			}
//			if ( position.x < ( gamemain.view_point_x + char_width ) )
//			{
//				position.x = ( gamemain.view_point_x + char_width );
//			}
//			if ( position.x > ( ( gamemain.view_point_x + gamemain.getScreenWidth( ) ) - char_width ) )
//			{
//				position.x = ( ( gamemain.view_point_x + gamemain.getScreenWidth( ) ) - char_width );
//			}
			break;
		case CHRTYPE.ENEMY1:
			DIRECTION old_direction = direction;	//前回の向き
			
			if ( move_buff[0] != 0 )
			{ 
				disable_input_count = 1;
				position.y = position.y - ( ( MOVE_SPEED / 2 ) * map_speed );
				if ( position.y < 3136 )
				{
					position.y = 3136;
					move_buff[0] = 0;		//地中から上ってくる時間
					disable_input_count = 0;
				}
			}
			if ( gamemain.powerup_count == 0 )
			{
				move_buff[2] = 0;		//パワーアップの終了
			}

			if ( disable_input_count == 0 )
			{

				//敵用仮想キーコード
/*
				Vartalkey_Up = 0;
				Vartalkey_Down = 0;
				Vartalkey_Left = 0;
				Vartalkey_Right = 0;
				Vartalkey_Botton1 = 0;
				Vartalkey_Botton2 = 0;
*/
				switch( enemy_seqense )
				{
				case 0:
					//敵用行動状態
//					enemy_seqense = 1;

					switch( param )
					{
					case 0:
						Vartalkey_Up = 0;
						Vartalkey_Down = 0;
						Vartalkey_Left = 0;
						Vartalkey_Right = 0;
						
						float islr;
						islr = player_position.x - position.x;
						if ( ( islr > 0 ) )
						{
							if ( ( gamemain.getMapWidth( ) / 2 ) < islr )
							{
								Vartalkey_Left = 1;
							}
							else 
							{
								Vartalkey_Right = 1;
							}
						}
						if ( ( islr < 0 ) )
						{
							if ( -( gamemain.getMapWidth( ) / 2 ) > islr )
							{
								Vartalkey_Right = 1;
							}
							else 
							{
								Vartalkey_Left = 1;
							}
						}
						if ( ( islr < 4 ) & ( ( islr > -4 ) ) )
						{
							Vartalkey_Right = 0;
							Vartalkey_Left = 0;
						}
						break;
					case 1:
						if ( move_buff[1] == 0 )
						{
							//プレイヤーを遠い方に進む
							Vartalkey_Up = 0;
							Vartalkey_Down = 0;
							Vartalkey_Left = 0;
							Vartalkey_Right = 0;
							move_buff[1] = 1;
							float islr;
							islr = player_position.x - position.x;
							if ( ( islr > 0 ) )
							{
								if ( ( gamemain.getMapWidth( ) / 2 ) < islr )
								{
									Vartalkey_Left = 1;
								}
								else 
								{
									Vartalkey_Right = 1;
								}
							}
							if ( ( islr < 0 ) )
							{
								if ( -( gamemain.getMapWidth( ) / 2 ) > islr )
								{
									Vartalkey_Left = 1;
								}
								else 
								{
									Vartalkey_Right = 1;
								}
							}
							if ( ( islr < 4 ) & ( ( islr > -4 ) ) )
							{
//								Vartalkey_Right = 0;
//								Vartalkey_Left = 0;
							}
						}
						break;
					case 2:
						if ( move_buff[1] == 0 )
						{
							Vartalkey_Up = 0;
							Vartalkey_Down = 0;
							Vartalkey_Left = 0;
							Vartalkey_Right = 0;
							move_buff[1] = 1;
							if ( Hell_randInt( 0, 2 ) == 0 )
							{
								Vartalkey_Right = 1;
							}
							else 
							{
								Vartalkey_Left = 1;
							}
						}
						break;
					default:
						break;
					}
					break;
				default:
					enemy_seqense = 0;
					break;
				}
				
				//プレイヤー
				// 移動方向決定
				if( ( keypad.isPressDown() & is_inputkey ) | Vartalkey_Down )
				{
					//下
//					position.y = position.y + 10;
				}
				else if( ( keypad.isPressUp() & is_inputkey ) | Vartalkey_Up )
				{
					//上
//					position.y = position.y - 10;
				}
				
				if( (keypad.isPressLeft() & is_inputkey ) | Vartalkey_Left )
				{
					//左
					direction = DIRECTION.LEFT;
					switch( motion )
					{
					case MOTION.NEUTRAL:	//入力なし
						motion = MOTION.WALKING;
					case MOTION.WALKING:	//歩き
						position.x = position.x - ( MOVE_SPEED * map_speed );
						break;
					case MOTION.JUMP_UP:		//ジャンプ上昇
					case MOTION.JUMP_DOWN:		//ジャンプ下降
						position.x = position.x - ( MOVE_SPEED * map_speed );
						break;
					case MOTION.DAMAGE:			//ダメージ
					case MOTION.DOWN:			//ゲームオーバー
					case MOTION.CLEAR:			//クリア
					case MOTION.ATTACK:			//地上攻撃
					case MOTION.JUMP_ATTACK:	//空中攻撃
					default:
						break;
					}
				}
				else if( (keypad.isPressRight() & is_inputkey ) | Vartalkey_Right )
				{
					//右
					direction = DIRECTION.RIGHT;
					switch( motion )
					{
					case MOTION.NEUTRAL:	//入力なし
						motion = MOTION.WALKING;
					case MOTION.WALKING:	//歩き
						position.x = position.x + ( MOVE_SPEED * map_speed );
						break;
					case MOTION.JUMP_UP:		//ジャンプ上昇
					case MOTION.JUMP_DOWN:		//ジャンプ下降
						position.x = position.x + ( MOVE_SPEED * map_speed );
						break;
					case MOTION.DAMAGE:			//ダメージ
					case MOTION.DOWN:			//ゲームオーバー
					case MOTION.CLEAR:			//クリア
					case MOTION.ATTACK:			//地上攻撃
					case MOTION.JUMP_ATTACK:	//空中攻撃
					default:
						break;
					}
				}
				else 
				{
					if ( place == PLACE.GROUND )
					{
						motion = MOTION.NEUTRAL;	//操作していないときはニュートラルにする
					}
				}
/*				
				if( ( keypad.isPushEnter() & is_inputkey ) | Vartalkey_Botton1 )
				{
					//１ボタン
					switch( motion )
					{
					case MOTION.NEUTRAL:	//入力なし
					case MOTION.WALKING:	//歩き
					case MOTION.JUMP_UP:		//ジャンプ上昇
					case MOTION.JUMP_DOWN:		//ジャンプ下降
					case MOTION.DAMAGE:			//ダメージ
					case MOTION.DOWN:			//ゲームオーバー
					case MOTION.CLEAR:			//クリア
					case MOTION.ATTACK:			//地上攻撃
					case MOTION.JUMP_ATTACK:	//空中攻撃
					default:
						break;
					}
				}
				if ( ( keypad.isPushCancel() & is_inputkey ) | Vartalkey_Botton2 )
				{
					//２ボタン
					if ( place == PLACE.GROUND )
					{
						jump_start( 5 );			//ジャンプ力の設定
						//地上にいる
						switch( motion )
						{
						case MOTION.NEUTRAL:	//入力なし
						case MOTION.WALKING:	//歩き
							motion = MOTION.JUMP_UP;
							break;
						case MOTION.JUMP_UP:		//ジャンプ上昇
						case MOTION.JUMP_DOWN:		//ジャンプ下降
						case MOTION.DAMAGE:			//ダメージ
						case MOTION.DOWN:			//ゲームオーバー
						case MOTION.CLEAR:			//クリア
						case MOTION.ATTACK:			//地上攻撃
						case MOTION.JUMP_ATTACK:	//空中攻撃
						default:
							break;
						}
					}
				}
*/
			}
			//前回と向きが違う場合
			if ( old_direction != direction )
			{
				move_buff[3] = 60;	//Uターンカウンタセット
			}
			if ( move_buff[3] > 0 )
			{
				move_buff[3]--;
			}
				
			if ( position.x < 0 )
			{
				position.x = position.x + gamemain.getMapWidth( );
			}
			if ( position.x >= gamemain.getMapWidth( ) )
			{
				position.x = position.x - gamemain.getMapWidth( );
			}
//			if ( position.x < ( gamemain.view_point_x + char_width ) )
//			{
//				position.x = ( gamemain.view_point_x + char_width );
//			}
//			if ( position.x > ( ( gamemain.view_point_x + gamemain.getScreenWidth( ) ) - char_width ) )
//			{
//				position.x = ( ( gamemain.view_point_x + gamemain.getScreenWidth( ) ) - char_width );
//			}
			break;
		case CHRTYPE.EFFECT1:
			//エフェクト
			move_buff[0]++;
			if ( move_buff[0] > 60 )
			{
				move_buff[0] = 0;
				life = 0;	//死亡
			} 
			break;
		case CHRTYPE.EFFECT2:
			//エフェクト
			move_buff[0]++;
			if ( move_buff[0] > 30 )
			{
				move_buff[0] = 0;
				life = 0;	//死亡
			} 
			position.x = position.x + ( Hell_cosEx( move_buff[6] ) * move_buff[5] );
			position.y = position.y + ( Hell_sinEx( move_buff[6] ) * move_buff[5] );
			break;
		case CHRTYPE.EFFECT3:
			//エフェクト
			move_buff[0]++;
			if ( move_buff[0] > 60 )
			{
				move_buff[0] = 0;
				life = 0;	//死亡
			} 
			move_buff[1]--;
			if ( move_buff[1] <= 0  )
			{
				move_buff[1] = 0;
			} 
			break;
		case CHRTYPE.EFFECT4:
			//エフェクト
			move_buff[0]++;
			if ( move_buff[0] > 60 )
			{
				move_buff[0] = 0;
				life = 0;	//死亡
			} 
			break;
		case CHRTYPE.MOB1:
			//にぎやかし
			if ( move_buff[0] > 0 )
			{
				move_buff[0]--;
			}
			if ( move_buff[3] == 0 )
			{
				move_buff[1] = move_buff[1] + 1;
				move_buff[2] = move_buff[2] - 1;
				if ( move_buff[1] > 110 )
				{
					move_buff[3] = 1;
				}
			}
			else 
			{
				move_buff[1] = move_buff[1] - 1;
				move_buff[2] = move_buff[2] + 1;
				if ( move_buff[1] < 90 )
				{
					move_buff[3] = 0;
				}
			}
			
			break;
		default:
			break;
		}
		
		switch( type )
		{
		case CHRTYPE.PLAYER1:
		case CHRTYPE.PLAYER2:
		case CHRTYPE.ENEMY1:
			if ( old_pos.x > position.x )			//前フレームのキャラタ座標
			{
				is_move_left( );
			}
			if ( old_pos.x < position.x )			//前フレームのキャラタ座標
			{
				is_move_right( );
			}
			
//			jump( );	//空中での座標移動
			
			break;
		default:
			break;
		}

		int x1 = ( cast( int )position.x + ( anime_width / 2 ) );
		int x2 = ( cast( int )position.x - ( anime_width / 2 ) );
		int y1 = ( cast( int )position.y + ( anime_height / 2 ) );
		int y2 = ( cast( int )position.y - ( anime_height / 2 ) );
		
		is_draw = 0;				//表示行うか
//		if (
//		     ( gamemain.view_point_x < x1 )
//		  && ( ( gamemain.view_point_x + gamemain.getScreenWidth( ) ) > x2 )
//		  && ( gamemain.view_point_y < y1 )
//		  && ( ( gamemain.view_point_y + gamemain.getScreenHeight( ) ) > y2 )
//		   )
//		{	
			is_draw_one = 1;			//画面内に１度でも表示を行ったか
			is_draw = 1;				//表示行うか
//		}
		
		if ( old_is_draw_one != is_draw_one )
		{
			switch( type )
			{
			case CHRTYPE.SCROLL_FREE:
				gamemain.set_scroll_status( gamemain.SCROLL_STATUS.SCROLL_FREE );
				break;
			case CHRTYPE.SCROLL_RIGHT:
				gamemain.set_scroll_status( gamemain.SCROLL_STATUS.SCROLL_RIGHT );
				break;
			case CHRTYPE.SCROLL_LEFT:
				gamemain.set_scroll_status( gamemain.SCROLL_STATUS.SCROLL_LEFT );
				break;
			case CHRTYPE.SCROLL_UP:
				gamemain.set_scroll_status( gamemain.SCROLL_STATUS.SCROLL_UP );
				break;
			case CHRTYPE.SCROLL_DOWN:
				gamemain.set_scroll_status( gamemain.SCROLL_STATUS.SCROLL_DOWN );
				break;
			case CHRTYPE.SCROLL_WAIT:
				gamemain.set_scroll_status( gamemain.SCROLL_STATUS.SCROLL_WAIT );
				break;
			case CHRTYPE.BG_SET:
				gamemain.set_bg( param );
				break;
			default:
				break;
			}
		}

		//ライフが0の場合の動作を行う
		is_life( );

	}
	
	
	//-----------------------------------------------------------------------------
	//	右移動
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void is_move_right( )
	{
		int px ;
		int py1 ;
		int py2 ;
		int bx ;
		int by ;

		int i;

		/* 今回の位置 */
		/* player[29]横バネ吹き飛び補正 */
		px = ( cast( int )position.x + char_width );
		py1 = ( cast( int )position.y - char_height );
		py2 = ( cast( int )position.y + char_height - 1 ); /* 足より1ドット高い場所を判定する */
		/* 画面外は一番上で判定 */ 
		if ( position.y <= 0 )
		{
			py1 = ( gamemain.getMapChipHeight( ) / 2 );
			py2 = py1;
		}

		if (
		     ( ( gamemain.get_map_id( px, py1 ) == gamemain.GROUND_TYPE.GROUND ) )
		  || ( ( gamemain.get_map_id( px, py2 ) == gamemain.GROUND_TYPE.GROUND ) ) 
		   )
		{
			position.x = ( ( px / gamemain.getMapChipWidth( ) ) * gamemain.getMapChipWidth( ) ) + ( ( gamemain.getMapChipWidth( ) / 2 ) - char_width );
		}
		else 
		{
		}
	}
	//-----------------------------------------------------------------------------
	//	左移動
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void is_move_left( )
	{
		int px ;
		int py1 ;
		int py2 ;
		int bx ;
		int by ;

		int i;

		/* 今回の位置 */
		/* player[29]横バネ吹き飛び補正 */
		px = ( cast( int )position.x - char_width );
		py1 = ( cast( int )position.y - char_height + 1 );
		py2 = ( cast( int )position.y + char_height - 1 ); /* 足より1ドット高い場所を判定する */
		/* 画面外は一番上で判定 */ 
		if ( position.y <= 0 )
		{
			py1 = ( gamemain.getMapChipHeight( ) / 2 );
			py2 = py1;
		}

		if (
		     ( ( gamemain.get_map_id( px, py1 ) == gamemain.GROUND_TYPE.GROUND ) )
		  || ( ( gamemain.get_map_id( px, py2 ) == gamemain.GROUND_TYPE.GROUND ) ) 
		   )
		{
			position.x = ( ( px / gamemain.getMapChipWidth( ) ) * gamemain.getMapChipWidth( ) ) + ( ( gamemain.getMapChipWidth( ) / 2 ) + char_width );
		}
		else 
		{
		}
	}
	
	//-----------------------------------------------------------------------------
	//	空中での座標移動
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void jump( )
	{
		int y1;
		int px ;
		int px1 ;
		int px2 ;
		int py ;
		int py3 ;
		int jump_move ;
		PLACE old_place = place;

		/* ジャンプフレームカウント */
		jump_fream_count++;
		if ( jump_fream_count >= JUMP_UPDATE_FREAM )
		{
			jump_fream_count = 0;
		}
		/* 今回の位置 */
		px = ( cast( int )position.x );
		px1 = ( cast( int )position.x - char_width + 1 );
		px2 = ( cast( int )position.x + char_width - 1 );
//		py = ( cast( int )position.y + char_height );
		py = ( cast( int )position.y + char_height + 1 );

		if ( place == PLACE.GROUND ) 	/* 地上 */
		{
			/* 100 ～ 200は地面 */
			/* 足元が地面ではない */
			if ( 
			     ( !( gamemain.get_map_id( px1, py ) == gamemain.GROUND_TYPE.GROUND ) )
			  && ( !( gamemain.get_map_id( px2, py ) == gamemain.GROUND_TYPE.GROUND ) ) 
			   )
			{
//				if ( item_up( ) != -1 )	/* アイテムの上に乗っている */
//				{
//					player[1] = item_up( ); 
//					player[14] = 10;
//					return;
//				}
				
				place = PLACE.AIR;		//空中
				jump_count = -1;			/* 落下開始 */
				
			}
			else
			{
				return;
			}
		}
		else 
		{
		}
		/* 10フレームに1回Y座標計算 */
		if ( jump_fream_count == 0 )	
		{
			jump_count = jump_count - JUMP_COUNT_MOVE;	/* ジャンプ力減少 */

			/* 頂点付近は-1落下開始とする */
			if ( ( jump_count <= 2 ) && ( jump_count > -2 ) )
			{
				jump_count = -1;
			}

			if ( jump_count < JUMP_COUNT_MIN )	/* 最速落下を設定 */
			{
				jump_count = -4;
			}
		}
		
		/* 移動量を計算 */
		jump_move = ( ( 0 - jump_count ) * ( 0 - jump_count ) * ( 0 - jump_count ) );
		position.y = position.y + ( jump_move / JUMP_SCALE );	/* Y移動 */
		
		/* 落下、上昇可能判定 */
		if ( ( position.y + char_height ) < 0 )
		{
			//画面上では行わない
			return;
		}
		
		py3 = ( cast( int )position.y - char_height );

		/* 落下 */
		if ( jump_count <= 0 )													/* 上り中は判定しない */
		{
			/* 地面判定 */
			if (
			     ( ( gamemain.get_map_id( px1, py ) == gamemain.GROUND_TYPE.GROUND ) )
			  || ( ( gamemain.get_map_id( px2, py ) == gamemain.GROUND_TYPE.GROUND ) ) 
			   )
			{
				/* 着地 */
//				if ( player[8] > 0 )	/* HPがある場合は効果音を鳴らす */
//				{ 
//					soundPlaySe( EN_SE_LANDING );
//					se_wav( 9 );
//				}
				place = PLACE.GROUND;		//地上
				position.y = ( ( py / gamemain.getMapChipHeight( ) ) * gamemain.getMapChipHeight( ) ) + ( ( gamemain.getMapChipHeight( ) / 2 ) - char_height ); 		/* 着地したマスのY座標に合わせる */

//				p_jump_f_set( player[0], player[1] );
			}
			else 
			{
//				if ( item_up( ) != -1 )						/* アイテムの上に乗っている */
//				{
//					soundPlaySe( EN_SE_LANDING );
//					se_wav( 9 );
//					if ( ( player[2] != 31 ) && ( player[2] != 32 ) )	/* ダメージ中じゃない */
//					{
//						player[2] = 0;							/* 着地 */
//						player[11] = 0;							/* アニメーションカウンタクリア */
//					}
//					player[1] = item_up( ); 
//					player[14] = 10;
//
//					player[18] = 0;			/* つかみからジャンプ */
//					player[28] = 0;			/* 流されている */
//
//					p_jump_f_set( player[0], player[1] );
//				}
			}
		}
		else		/* 上昇中 */
		{
			/* 地面判定 */
			if (
			     ( ( gamemain.get_map_id( px1, py3 ) == gamemain.GROUND_TYPE.GROUND ) )
			  || ( ( gamemain.get_map_id( px2, py3 ) == gamemain.GROUND_TYPE.GROUND ) ) 
			   )
			{
				/* 頭がぶつかった */
				jump_count = -1;		/* 落下 */
				position.y = ( ( py3 / gamemain.getMapChipHeight( ) ) * gamemain.getMapChipHeight( ) ) + ( ( gamemain.getMapChipHeight( ) / 2 ) + char_height ); 		/* 着地したマスのY座標に合わせる			}
			else 
			{
//				if ( item_up2( ) != -1 )	/* アイテムに頭がぶつかる */
//				{
//					player[1] = item_up2( ); 
//					player[13] = 0;
//					player[14] = -3;
//				}
			}
		}
		
		/* 画面外、下に行った */
		if ( ( position.y ) > ( gamemain.getMapHeight()) )
		{
			position.y = gamemain.getMapHeight();
//			if ( map_data[2] == 0 )		/* スクロールしない場合はダメージ */
//			{
//				if ( player[8] > 0 )			/* HPがある */
//				{ 
//					if ( player[17] == 0 )	/* 無敵時間が無い */
//					{
//						player[17] = 120;	/* 無敵時間 */
//						player[8] = player[8] - 1;/* ダメージ */
//						se_wav( 1 );	/* ダメージ効果音 */
//					}
//					if ( player[8] > 0 )		/* HPがある */
//					{ 
//						player[0] = player[15];	/* リトライ位置に移動 */
//						player[1] = player[16];
//						wait_scl2( save_data[3] ,player[0], player[1] );	/* 暗転戻り */
//						
//	//					player[13] = 0;
//	//					player[14] = 6;
//					}
//					else 
//					{
//						player[1] = -14;
//					}
//				}
//				else
//				{
//					player[1] = -14;
//				}
//			}
		}
		
		//空中モーションの制御
		if ( place != old_place )
		{
			jump_fream_count = 0;
			
			switch ( place )
			{
			case PLACE.GROUND:
				if ( 
				     ( motion == MOTION.DAMAGE )		/* ダメージ中 */
				  || ( motion == MOTION.JUMP_ATTACK )	/* 空中攻撃中 */
				   )
				{
				}
				else 
				{
					motion = MOTION.NEUTRAL;		/* ニュートラル */
				}
				break;
			case PLACE.AIR:
				/* 歩き中なら空中状態へ移行 */
				if ( ( motion == MOTION.NEUTRAL )
				  || ( motion == MOTION.WALKING ) )
				{
					motion = MOTION.JUMP_UP;		/* 空中 */
				}
				if ( motion == MOTION.DAMAGE )	/* ダメージ中 */
				{
				}
				break;
			default:
				break;
			}
		}
		if ( place == PLACE.AIR )
		{
			/* 落下 */
			if ( jump_count < -2 )													/* 上り中は判定しない */
			{
				motion = MOTION.JUMP_DOWN;		/* 空中 */
			}
		}
	}
	
	//-----------------------------------------------------------------------------
	//	空中での座標移動
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void jump_start( int st )
	{
		jump_count = st;			//ジャンプカウンタ
		jump_fream_count = 0;		//ジャンプフレームカウンタ
		place = PLACE.AIR;			//空中
	}
	

	//-----------------------------------------------------------------------------
	//	当たり判定で呼び出される
	//-----------------------------------------------------------------------------
	//	引	数: int chartype：接触したキャラクターのタイプ
	//			int param1：汎用パラメータ１
	//			int param2：汎用パラメータ２
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void hit( int chartype, Character hitchar, int param1 )
	{
		
		
		switch( type )
		{
		case CHRTYPE.PLAYER1:
		case CHRTYPE.PLAYER2:
			switch( chartype )
			{
			case CHRTYPE.ENEMY1:
				if ( hitchar.move_buff[2] != 0 )
				{
					Hell_playWAV( "destroy", 0, -1 );
					//スコアの加算
					gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_SCOER_UP_ENEMYGET, 0, 0, 0, 0, 0 );
					//エフェクトの生成
					gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_EFFECT2, position.x, position.y, 0, 0, 0 );
					//敵の生成
					gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_ENEMY_CREATE, 0, 0, 0, hitchar.param, 0 );
					gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_ENEMY_GETWAIT, 0, 0, 0, hitchar.param, 0 );
					hitchar.destroy( );
				}
				else 
				{
					Hell_playWAV( "damage", 0, -1 );
					//スコア倍率のクリア
					gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_SCOER_COUNT_CLAER, 0, 0, 0, 0, 0 );
					//敵キャラクターの削除
					gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_ENEMY_DEL, 0, 0, 0, 0, 0 );
					
					//アイテムの消去
					move_buff[2] = -1;
					move_buff[3] = -1;
					move_buff[4] = -1;
					
					hitchar.move_buff[4] = 1;
					disable_input_count = 30;
					motion = MOTION.DAMAGE;
				}
				break;
			case CHRTYPE.ITEM1:
				if ( hitchar.move_buff[0] == 0 )
				{
					move_buff[4] = move_buff[3];
					move_buff[3] = move_buff[2];
					move_buff[2] = hitchar.param;
				}
				//アイテムの取得を判定しパワーアップをする
				if ( 
				     ( move_buff[2] != -1 )
				  && ( move_buff[3] != -1 )
				  && ( move_buff[4] != -1 )
				   )
				{
					//3つ取得した
					if ( 
					     ( move_buff[2] != move_buff[3] )
					  && ( move_buff[3] != move_buff[4] )
					  && ( move_buff[4] != move_buff[2] )
					   )
					{
						//パワーアップを行う
						no_hit_count = 2 * map_speed;	//無敵時間の設定 パワーアップ状態に移行するまで敵と当たり判定を行わない
					}
				}
				break;
			case CHRTYPE.ITEM2:
				move_buff[0] = 1;	//ワープ可能
				break;
			default:
				break;
			}
			break;
		case CHRTYPE.ITEM1:
			switch( chartype )
			{
			case CHRTYPE.PLAYER1:
			case CHRTYPE.PLAYER2:
				if ( move_buff[0] == 0 )
				{
					Hell_playWAV( "get", 0, -1 );
					//スコアの加算
					gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_SCOER_UP_ITEMGET, 0, 0, 0, 0, 0 );
					//エフェクトの生成
					gamemain.character_reqest( gamemain.REQEST_CODE.REQEST_EFFECT2, position.x, position.y, 0, 0, 0 );
					move_buff[0] = 1;	//取得された
				}
				break;
			default:
				break;
			}
			break;
		case CHRTYPE.ITEM2:
			switch( chartype )
			{
			case CHRTYPE.PLAYER1:
			case CHRTYPE.PLAYER2:
				move_buff[0] = 1;	//プレイヤーがドアに重なっている
				break;
			default:
				break;
			}
			break;
		default:
			break;
		}
		
	}


	//-----------------------------------------------------------------------------
	//	当たり判定で呼び出される
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void is_life( )
	{
		if ( life <= 0 )
		{
			destroy( );
		}
	}


	//-----------------------------------------------------------------------------
	//	削除
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void destroy( )
	{
		type = CHRTYPE.NONE;
		exist = false;
	}


	//-----------------------------------------------------------------------------
	//	描画
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void draw( )
	{
		if ( is_draw == 0 )
		{
			//画面外
			return;
		}
		
		//位置から円の位置を計算する
		float rot_scale_x;
		rot_scale_x = ( position.x ) / cast( float )gamemain.getMapWidth( );
		rot_scale_x = rot_scale_x * 360;
		disp_rotate = map_rotate - rot_scale_x;
		
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		switch( type )
		{
		case CHRTYPE.PLAYER1:
		case CHRTYPE.PLAYER2:
			//プレイヤー
			int reverse = 1;				//反転1or-1
			int reverse_offset_x = 0;				//反転1or-1
			int reverse_height = 0;			//反転1or-1
			if ( direction == DIRECTION.LEFT )
			{
				if( anime_reverse != 0 )	//反転あり
				{
					reverse = reverse * -1;
					reverse_offset_x= anime_width;
				}
				else 
				{
					reverse_height = anime_height;
				}
			}
			//画面内の場合に表示
/*
			Hell_drawTexture("char"
				, position.x - ( anime_width / 2 ) - gamemain.view_point_x
				, position.y - ( anime_height / 2 ) - gamemain.view_point_y
				, anime_start_x + ( anime_pattern_count * anime_width )
				, anime_start_y + reverse_height
				, anime_width , anime_height
				, reverse , 1.0 );
*/
/*
			disp_rotate = 0;
			Hell_drawTexture("char"
				, 320 - ( anime_width / 2 )
				, ( 70 - ( 3136 - position.y ) ) - ( anime_height / 2 )
				, anime_start_x + ( anime_pattern_count * anime_width )
				, anime_start_y + reverse_height
				, anime_width , anime_height
				, reverse , 1.0
				,disp_rotate );
*/
			disp_rotate = 0;
			//ワープ中は表示しない
			if ( gamemain.iSceneStatus != gamemain.SCENE.PLAYER_WARP )
			{
				Hell_drawTexture("char"
					, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( anime_width / 2 ) + reverse_offset_x
					, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( anime_height / 2 )
					, anime_start_x + ( anime_pattern_count * anime_width )
					, anime_start_y + reverse_height
					, anime_width , anime_height
					, reverse , 1.0
					,disp_rotate
					,255, 255, 255, 255 );
			}
			//取得アイテムの表示
			if ( gamemain.iSceneStatus != gamemain.SCENE.CLEAR )
			{
				if ( move_buff[2] != -1 )
				{
					//画面内の場合に表示
					Hell_drawTexture("item"
						, 260
						, 20
						, 0 + ( move_buff[2] * 32 )
						, 0
						, 32 , 32
						, 1.0 , 1.0
						, 0 );
				}
				if ( move_buff[3] != -1 )
				{
					//画面内の場合に表示
					Hell_drawTexture("item"
						, 300
						, 20
						, 0 + ( move_buff[3] * 32 )
						, 0
						, 32 , 32
						, 1.0 , 1.0
						, 0 );
				}
				if ( move_buff[4] != -1 )
				{
					//画面内の場合に表示
					Hell_drawTexture("item"
						, 340
						, 20
						, 0 + ( move_buff[4] * 32 )
						, 0
						, 32 , 32
						, 1.0 , 1.0
						, 0 );
				}
			}
	

			break;
		case CHRTYPE.ENEMY1:
			//敵
			int color_r;
			int color_g;
			int color_b;
			
			int reverse = 1;				//反転1or-1
			int reverse_offset_x = 0;				//反転1or-1
			int reverse_height = 0;			//反転1or-1
			int reverse_size = 0;			//反転1or-1
			
			switch ( param )
			{
			case 0:
				color_r = 255;
				color_g = 200;
				color_b = 200;
				break;
			case 1:
				color_r = 200;
				color_g = 255;
				color_b = 200;
				break;
			case 2:
				color_r = 200;
				color_g = 200;
				color_b = 255;
				break;
			default:
				color_r = 255;
				color_g = 255;
				color_b = 255;
				break;
			}
			//パワーアップ中
			if ( move_buff[2] != 0 )
			{
				//戻る１秒前に点滅をする
				if (
				     ( gamemain.powerup_count < 180 )
				  && ( ( cast( int )( gamemain.powerup_count / 20 ) % 2 ) == 1 )
				   )
				{
				}
				else 
				{
//					color_r = 60;
//					color_g = 60;
//					color_b = 170;
				}
				color_r = 60;
				color_g = 60;
				color_b = 170;
			} 
			
			if ( direction == DIRECTION.LEFT )
			{
				if( anime_reverse != 0 )	//反転あり
				{
					reverse = reverse * -1;
					reverse_offset_x = anime_width;
				}
				else 
				{
					reverse_height = anime_height;
				}
			}
			//画面内の場合に表示
			Hell_setAlpha(HELL_ALPHA_ADD);
//			Hell_setAlpha(HELL_ALPHA_NORMAL);
			float srr = 1.2;
			Hell_drawTexture("enemy"
				, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( ( cast( float )anime_width * srr ) / 2 ) + ( reverse_offset_x * srr )
				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( ( cast( float )anime_height * srr ) / 2 )
				, anime_start_x + ( anime_pattern_count * anime_width )
				, anime_start_y + reverse_height
				, anime_width , anime_height
				, ( reverse * srr ), srr
				,disp_rotate
				,color_r, color_g, color_b, 255 );
			Hell_setAlpha(HELL_ALPHA_NORMAL);
			Hell_drawTexture("enemy"
				, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( anime_width / 2 ) + reverse_offset_x
				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( anime_height / 2 )
				, anime_start_x + ( anime_pattern_count * anime_width )
				, anime_start_y + reverse_height
				, anime_width , anime_height
				, reverse , 1.0
				,disp_rotate
				,color_r, color_g, color_b, 255 );
//			if ( 
//			     ( ( move_buff[3] > 0 ) && ( move_buff[3] <= 30 ) && ( ( move_buff[3] % 2 ) == 0 ) )	//Uターンをした
//			  || ( ( move_buff[3] > 30 ) && ( move_buff[3] <= 60 ) && ( ( move_buff[3] % 3 ) == 0 ) )	//Uターンをした
//			   )
			if ( move_buff[3] > 0 )	//Uターンをした
			{
				Hell_setAlpha(HELL_ALPHA_ADD);
				Hell_drawTexture("enemy"
					, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( anime_width / 2 ) + reverse_offset_x
					, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( anime_height / 2 )
					, anime_start_x + ( anime_pattern_count * anime_width )
					, anime_start_y + reverse_height
					, anime_width , anime_height
					, reverse , 1.0
					,disp_rotate
					,color_r, color_g, color_b, ( move_buff[3] * 4 ) );
//					,255, 255, 255, ( move_buff[3] * 4 ) );
			}
			//プレイヤーにあたった
			if ( move_buff[4] != 0 )
			{
				Hell_setAlpha(HELL_ALPHA_ADD);
				Hell_drawTexture("enemy"
					, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( anime_width / 2 ) + reverse_offset_x
					, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( anime_height / 2 )
					, anime_start_x + ( anime_pattern_count * anime_width )
					, anime_start_y + reverse_height
					, anime_width , anime_height
					, reverse , 1.0
					,disp_rotate
//					,color_r, color_g, color_b, ( move_buff[3] * 50 ) );
					,255, 255, 255, 200 );
			} 

			break;
		case CHRTYPE.ITEM1:
			int r = 255;
			int g = 255;
			int b = 255;
			int a = 255;
			//アイテム　取得
			//画面内の場合に表示
			Hell_setAlpha(HELL_ALPHA_ADD);
			Hell_drawTexture("item"
				, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( ( 32 * 1.2 ) / 2 )
				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( ( 32 * 1.2 ) / 2 )
				, 0 + ( param * 32 )
				, 0 + ( ( type - CHRTYPE.ITEM_START ) * 32 )
				, 32 , 32
				, 1.2 , 1.2
				,disp_rotate );
			Hell_setAlpha(HELL_ALPHA_NORMAL);
			Hell_drawTexture("item"
				, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( 32 / 2 )
				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( 32 / 2 )
				, 0 + ( param * 32 )
				, 0 + ( ( type - CHRTYPE.ITEM_START ) * 32 )
				, 32 , 32
				, 1.0 , 1.0
				,disp_rotate );
				
			//プレイヤーの状態と見てメッセージの表示
			if ( gamemain.iSceneStatus != gamemain.SCENE.CLEAR )
			{
				int ch_idx = gamemain.get_character_index( cast( int )CHRTYPE.PLAYER1, 0 );
				if ( ch_idx != -1 )
				{
					if ( 
					     ( gamemain.character[ch_idx].move_buff[2] != -1 )
					  && ( gamemain.character[ch_idx].move_buff[3] != -1 )
					   )
					{
						//アイテムボーナスを取れる場合
						if ( gamemain.character[ch_idx].move_buff[2] == gamemain.character[ch_idx].move_buff[3] )
						{
							//自分の種類がアイテムボーナスが取れるアイテムであった
							if ( ( param ) == gamemain.character[ch_idx].move_buff[2] )
							{
								//同じアイテムを３つ取得した
								Hell_drawFont( "NEXT BONUS"
									, cast( int )( 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 150 + ( 3136 - position.y ) ) ) - ( ( 10 * 1.0 ) / 2 ) - 40 )
									, cast( int )( 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 150 + ( 3136 - position.y ) ) ) - ( ( 10 * 1.0 ) / 2 ) )
									, 1, r,g,b );
							}
						}
						//アイテムボーナスを取れる場合
						if ( gamemain.character[ch_idx].move_buff[2] != gamemain.character[ch_idx].move_buff[3] )
						{
							//自分の種類がアイテムボーナスが取れるアイテムであった
							if ( 
							     ( ( param ) != gamemain.character[ch_idx].move_buff[2] )
							  && ( ( param ) != gamemain.character[ch_idx].move_buff[3] )
							   )
							{
								//同じアイテムを３つ取得した
								Hell_drawFont( "NEXT POWER UP"
									, cast( int )( 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 150 + ( 3136 - position.y ) ) ) - ( ( 10 * 1.0 ) / 2 ) - 50 )
									, cast( int )( 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 150 + ( 3136 - position.y ) ) ) - ( ( 10 * 1.0 ) / 2 ) )
									, 1, r,g,b );
							}
						}
					}
				}
			}	
				
			break;
		case CHRTYPE.ITEM2:
			//アイテム ドア
			//画面内の場合に表示
			Hell_drawTexture("item"
				, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( 32 / 2 )
				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( 32 / 2 )
				, 0 + ( param * 32 )
				, 0 + ( ( type - CHRTYPE.ITEM_START ) * 32 )
				, 32 , 32
				, 1.0 , 1.0
				,disp_rotate );
			if ( move_buff[0] != 0 )	//プレイヤーがドアに重なっている
			{
				Hell_setAlpha(HELL_ALPHA_ADD);
				Hell_drawTexture("item"
					, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( 32 / 2 )
					, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( 32 / 2 )
					, 0 + ( param * 32 )
					, 0 + ( ( type - CHRTYPE.ITEM_START ) * 32 )
					, 32 , 32
					, 1.0 , 1.0
					,disp_rotate );
			}
			break;
		case CHRTYPE.EFFECT1:
			//アイテム出現エフェクト
			//飛び散るエフェクト
			//画面内の場合に表示
			Hell_setAlpha(HELL_ALPHA_ADD);
			int disp_size = cast( int )( 128 * ( cast( float )( 60 - move_buff[0] ) / cast( float )60 ) );
			Hell_drawTexture("effect"
				, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( disp_size / 2 )
//				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 170 - ( 3136 - position.y ) ) ) - ( disp_size / 2 )
				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( disp_size / 2 )
				, 768
				, 0
				, 128 , 128
				, ( cast( float )( 60 - move_buff[0] ) / 60.0f ) , ( cast( float )( 60 - move_buff[0] ) / 60.0f )
				,disp_rotate
				,255, 255, 255, 200 - ( move_buff[0] * 3 ) );
			break;
		case CHRTYPE.EFFECT2:
			//アイテム出現エフェクト
			//画面内の場合に表示
			Hell_setAlpha(HELL_ALPHA_ADD);
			int disp_size = cast( int )( 128 * ( cast( float )( 30 - move_buff[0] ) / cast( float )30 ) / 2 );
			Hell_drawTexture("effect"
				, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( disp_size / 2 )
				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( disp_size / 2 )
				, 768
				, 0
				, 128 , 128
				, ( cast( float )( 30 - move_buff[0] ) / 30.0f ) / 2 , ( cast( float )( 30 - move_buff[0] ) / 30.0f ) / 2
				,disp_rotate
				,255, 255, 255, 200 - ( move_buff[0] * 3 ) );
			break;
		case CHRTYPE.EFFECT3:
			//取得スコア表示
			int r = 255;
			int g = 255;
			int b = 255;
			int a = 255;
			//画面内の場合に表示
			switch( move_buff[2] )
			{
			case 0:
				//表示なし
				break;
			case 1:
				//通常アイテム取得
				r = 200;
				g = 200;
				b = 255;
				Hell_drawFont( "ITEM GET"
				             , 260
				             , 100
				             , 2, r,g,b );
				break;
			case 2:
				r = 200;
				g = 200;
				b = 255;
				//同じアイテムを３つ取得した
				Hell_drawFont( "SAME ITEM3 GET"
				             , 260
				             , 100
				             , 2, r,g,b );
				break;
			case 3:
				r = 255;
				g = 200;
				b = 200;
				//敵を倒した
				Hell_drawFont( "ENEMY GET"
				             , 260
				             , 100
				             , 2, r,g,b );
				break;
			case 4:
				r = 200;
				g = 255;
				b = 200;
				//走り続けた
				Hell_drawFont( "NON STOP RUN"
				             , 260
				             , 100
				             , 2, r,g,b );
				break;
			case 5:
				r = Hell_randInt( 127, 255 );
				g = Hell_randInt( 127, 255 );
				b = Hell_randInt( 127, 255 );
				//タイムエクステンド
				Hell_drawFont( "TIME EXTEND"
				             , 198
				             , 250
				             , 3, r,g,b );
				char[] str;
				str = std.string.format( "%02d SEC", ( param ) );
				Hell_drawFont( str , 230, 290 , 4 , 255,255,255);
				break;
			default:
				break;
			}
			if ( move_buff[2] != 5 )
			{
				Hell_drawFont( std.string.toString( cast( int )param )
				             , 290 - ( ( ( 10 - move_buff[1] ) / 3 ) * 10 )
				             , 70 + ( ( 100 - ( move_buff[1] * move_buff[1] ) ) / 2 )
				             , 1 + ( ( 10 - move_buff[1] ) / 3 ), r,g,b );
			}
			break;
		case CHRTYPE.MOB1:
			//にぎやかし
			if ( ( move_buff[5] ) > 300 )               /*【変更01_002】明暗制御カウンタ(move_buff[5])の数値を1減らす(300になるまで減り続ける)*/
			{                                           /*【変更01_003】明暗制御カウンタ(move_buff[5])の数値を1減らす(300になるまで減り続ける)*/
				move_buff[5]--;                         /*【変更01_004】明暗制御カウンタ(move_buff[5])の数値を1減らす(300になるまで減り続ける)*/
			}                                           /*【変更01_005】明暗制御カウンタ(move_buff[5])の数値を1減らす(300になるまで減り続ける)*/
//			int r = 64;
//			int g = 64;
//			int b = 64;
//			int a = 255;
			int r = cast( int )move_buff[5] / cast( int )10;   /*【変更01_006】明暗の設定(最終値30)*/
			int g = cast( int )move_buff[5] / cast( int )10;   /*【変更01_007】明暗の設定(最終値30)*/
			int b = cast( int )move_buff[5] / cast( int )10;   /*【変更01_008】明暗の設定(最終値30)*/
			int a = 255;
			
			if ( gamemain.powerup_count > 0 )
			{
				a = 15;
				if ( gamemain.powerup_count < 60 )
				{
					a = 15 + cast( int )( 60 - gamemain.powerup_count ) * 4 ;
				}
			}
			if ( gamemain.iSceneStatus == gamemain.SCENE.CLEAR )
			{
				r = 255;
				g = 255;
				b = 255;
				a = 255;
			}
			
//			float w_r = cast( float )move_buff[1] + cast( float )( move_buff[4] * 10 );
//			float h_r = cast( float )move_buff[2] + cast( float )( move_buff[4] * 10 );
//			float w_r = 100 + cast( float )( move_buff[4] * 10 );
//			float h_r = 100 + cast( float )( move_buff[4] * 10 );
			float w_r = cast( float )move_buff[1];
			float h_r = cast( float )move_buff[2];

			//画面内の場合に表示
			int disp_sizex = cast( int )( 128 * ( cast( float )( w_r ) / cast( float )100 ) );
//			int disp_sizey = cast( int )( 128 * ( cast( float )( h_r ) / cast( float )100 ) );
			int disp_sizey = cast( int )( 256 * ( cast( float )( h_r ) / cast( float )100 ) );                /*【変更02_0001】Y座標の移動幅を256(２倍)に変更*/
			Hell_drawTexture("effect"
//				, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( ( 128 * move_buff[4] * 10 ) / 100 / 2 ) + ( 3136 - position.y ) ) ) - ( disp_sizex / 2 )
//				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( ( 128 * move_buff[4] * 10 ) / 100 / 2 ) + ( 3136 - position.y ) ) ) - ( disp_sizey / 2 )
				, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) * 2 / 3 - ( disp_sizex / 2 )    /*【変更02_0002】にぎやかし(建物)の描画位置をやや内側になるように設定*/
				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) * 2 / 3 - ( disp_sizey / 2 )    /*【変更02_0003】にぎやかし(建物)の描画位置をやや内側になるように設定*/
//				, 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( disp_sizex / 2 )
//				, 240 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( disp_sizey / 2 )
				, 0 + ( param * 128 )
				, 0 + ( move_buff[0] * 128 )
				, 128 , 256               /*【変更02_0004】画像の読込範囲を128x256（縦方向に2倍）のサイズに変更*/
//				, 128 , 128
				, cast( float )w_r / cast( float )100, cast( float )h_r / cast( float )100
				,disp_rotate
				,r, g, b, a );
			break;
		default:
			break;
		}
/*
Hell_setAlpha(HELL_ALPHA_NORMAL);
Hell_drawFont( std.string.toString( cast( int )position.x )
//Hell_drawFont( std.string.toString( cast( int )move_buff[2] )
             , cast( int )( 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( 32 / 2 ) )
             , cast( int )( 260 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( 32 / 2 ) )
             , 1 , 255,255,255 );
Hell_drawFont( std.string.toString( cast( int )position.y )
             , cast( int )( 320 + ( Hell_cosEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( 32 / 2 ) )
             , cast( int )( 270 - ( Hell_sinEx( map_rotate + 90 - rot_scale_x ) * ( 172 + ( 3136 - position.y ) ) ) - ( 32 / 2 ) )
             , 1 , 255,255,255 );
*/
/*
Hell_drawTexture("char"
	, position.x - ( 32 / 2 ) - gamemain.view_point_x
	, position.y - ( 32 / 2 ) - gamemain.view_point_y
	, 0
	, 0
	, 32 , 32
	, 1.0 , 1.0 );
*/
// for debug code end

	}

	//-----------------------------------------------------------------------------
	//	フレーム開始処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void freamStart( )
	{
		switch( type )
		{
		case CHRTYPE.PLAYER1:
		case CHRTYPE.PLAYER2:
			old_motion = motion;						//前フレームのモーション
			move_buff[0] = 0;	//ワープに重なった
			move_buff[6] = 0;	//今回のフレームは移動を行ったか？
			break;

		case CHRTYPE.ENEMY1:
			move_buff[4] = 0;		//プレイヤーにあたった
			old_motion = motion;						//前フレームのモーション
			break;

		case CHRTYPE.ITEM1:
			break;
		case CHRTYPE.ITEM2:
			move_buff[0] = 0;	//プレイヤーがドアに重なっている
			break;

		default:
			break;
		}
	}
	//-----------------------------------------------------------------------------
	//	フレーム終了処理
	//-----------------------------------------------------------------------------
	//	引	数: なし
	//	戻り値: なし
	//-----------------------------------------------------------------------------
	void freamEnd( )
	{
		switch( type )
		{
		case CHRTYPE.PLAYER1:
		case CHRTYPE.PLAYER2:
			if ( old_motion != motion )
			{
				anime_pattern_count = 0;
				anime_frame_count = 0;
				motiondata_set( );
			}

			if ( move_buff[0] == 1 )	//ワープに重なった
			{
				move_buff[1] = 1;	//ワープ可能
			}
			else 
			{
				move_buff[1] = 0;	//ワープ可能
			}
			break;
		case CHRTYPE.ENEMY1:
			if ( old_motion != motion )
			{
				anime_pattern_count = 0;
				anime_frame_count = 0;
				motiondata_set( );

				char_width_start = char_width;	//基準となるあたり判定を保存
			}
			char_width = cast( int )( char_width_start * map_speed );	//ゲームスピードにあわせて当たり判定を広くする

			if ( gamemain.itemget != 0 )	//プレイヤーがアイテムを取得した
			{
				if ( param == 2 )	//方向転換を行う
				{
					move_buff[1] = 0;
				}
				if ( param == 1 )	//方向転換を行う
				{
					if ( Hell_randInt( 0, 2 ) == 0 )
					{
						move_buff[1] = 0;
					}
				}
			}
			break;

		case CHRTYPE.ITEM1:
		case CHRTYPE.ITEM2:
			break;

		default:
			break;
		}
	}
}


