﻿/********************************************************************************/
/* 	state.d																		*/
/*------------------------------------------------------------------------------*/
/*	製作		D.K（あるふぁ～秘密基地）										*/
/*	製作開始	2008/05/22														*/
/*	MAIL		dk@red.interq.or.jp												*/
/*	URL			http://maglog.jp/alpha-secret-base/								*/
/*																				*/
/*	このソースは「やわらかライセンス」の元で配布されています。					*/
/*-更新履歴---------------------------------------------------------------------*/
/*	2008/05/22	初版															*/
/*-その他-----------------------------------------------------------------------*/
/*	なし																		*/
/********************************************************************************/

/* ライブラリのインポート *******************************************************/
private import hell2;		// dHell2ライブラリ
private import vector;
private import keypad;
private import actor;
private import clickable;
private import gamecore;

private import std.string;
private import std.math;
private import std.math2;
private import std.stream;
private import std.thread;

import config;

/* グローバル定数定義 ***********************************************************/
GameState g_gamestate = null;


/* load *************************************************************************/
/*	リソースの読み込み															*/
/*-引数-------------------------------------------------------------------------*/
/*	なし																		*/
/*-返り値-----------------------------------------------------------------------*/
/*	なし																		*/
/********************************************************************************/
void load()
{
	Hell_loadTexture("font","resource/font/font.bmp" , [255,255,255]);
	Hell_loadTexture("menu","resource/image/menu.bmp" , [0,255,0]);
	
	Hell_write("resource loaded.\n");
}

/* GameState ********************************************************************/
/*	ゲーム全体の状態遷移を管理するクラス										*/
/********************************************************************************/
class GameState
{
private:
	int msg;				// リクエストメッセージ
public:
	enum
	{
		MSG_NONE = 0,
		MSG_REQ_LOGO,			// ロゴ状態へ　行きたい
		MSG_REQ_TITLE,			// タイトル状態へ　行きたい
		MSG_REQ_GAME,			// ゲーム
		
		MSG_EXIT,				//アプリケーションの終了
	}
	
	void move(){}
	void draw(){}
	
	// 設定
	void setMsg(int m)
	{
		msg = m;
	}
	// 取得
	int getMsg()
	{
		return msg;
	}
}



bool loop()
{
	bool is_exit = true;	//アプリケーションの終了
	if(g_gamestate)
	{
		// メッセージ指示による状態遷移
		switch(g_gamestate.msg)
		{
			case GameState.MSG_NONE:
				break;
			case GameState.MSG_REQ_LOGO:
				g_gamestate = new LogoMain();
				break;
			case GameState.MSG_REQ_TITLE:
				g_gamestate = new TitleMain();
				break;
			case GameState.MSG_REQ_GAME:
				g_gamestate = new GameMain();
				break;
			case GameState.MSG_EXIT:
			default:
				is_exit = false;
				Hell_write("Unknown msg at loop.");
		}
	}
	else
	{
		// 初期状態
		g_gamestate = new LogoMain();
		Hell_write("Exec loop.\n");
	}
	
	g_gamestate.move();
	g_gamestate.draw();
	
	return( is_exit );
}

/* TitleMain ********************************************************************/
/*	タイトルの状態																*/
/********************************************************************************/

class TitleMain : GameState
{
public:
	int timer;
	Keypad keypad;					// キーボード/ゲームパッド取得用
	int init_flag;
	ConfigParser savedata;			// セーブデータ
	int hi_score[10];
	int star;
	int select;
	int mode;
	int maxstar;
	
	this()
	{
Hell_write( "TitleMain\n" );
		setMsg( GameState.MSG_NONE );
		timer = 0;
		init_flag = 0;
		star = 0;
		mode = 0;
		select = 0;

		keypad = new Keypad();
		//セーブデータ
		hi_score[0] = 0;
		hi_score[1] = 0;
		hi_score[2] = 0;
		hi_score[3] = 0;
		savedata = new ConfigParser();
		try
		{
			savedata.load(CONFIG_PATH);
		}
		catch(Exception e)
		{
			savedata.set("HOSHI",         14);
			savedata.set("SELECT_HOSHI",         0);
			savedata.set("SELECT_MODE",         0);
			savedata.set("HI_SCORE",         0);
			savedata.set("HI_SCORE_TIME",         0);
			savedata.set("HI_SCORE_ARCADE",         0);
			savedata.write();
		}
		hi_score[0] = savedata.getint("HI_SCORE");
		hi_score[1] = savedata.getint("HI_SCORE_TIME");
		hi_score[2] = savedata.getint("HI_SCORE_ARCADE");
		maxstar = savedata.getint("HOSHI");
		star = savedata.getint("SELECT_HOSHI");
		mode = savedata.getint("SELECT_MODE");
		
	}
	
	void move()
	{
		if ( init_flag == 0 )
		{
			init_flag = 1;
			Hell_loadTexture("title","resource/image/title.bmp" , [0,255,0]);
			Hell_playBgm( "resource/sound/title.ogg", -1, 0 );
		}
		timer++;

		if ( keypad.isPushDown() )
		{
			//下
			select++;
			if ( select > 1 )
			{
				select = 1;
			} 
		}
		else if ( keypad.isPushUp() )
		{
			//上
			select--;
			if ( select < 0 )
			{
				select = 0;
			} 
		}
		
		if(keypad.isPushEnter())
		{
			//１ボタン
			savedata.set("SELECT_HOSHI",         star);
			savedata.set("SELECT_MODE",          mode);
			savedata.write();

			Hell_SetGameflag( 0, 0 );	//リプレイモードOFF
			Hell_SetGameflag( 1, star );	//星番号
			
			switch ( mode )
			{
			case 0:
				Hell_SetGameflag( 2, 0 );	//タイムエクステンドモード OFF
				break;
			case 1:
				Hell_SetGameflag( 2, 1 );	//タイムエクステンドモード ON
				break;
			case 2:
				Hell_SetGameflag( 2, 2 );	//アーケードモード
				break;
			default:
				break;
			}
			setMsg(GameState.MSG_REQ_GAME);
		}
		if(keypad.isPushCancel())
		{
			//2ボタン
			savedata.set("SELECT_HOSHI",         star);
			savedata.set("SELECT_MODE",          mode);
			savedata.write();

			Hell_SetGameflag( 0, 1 );	//リプレイモードON
			Hell_SetGameflag( 1, star );	//星番号
			switch ( mode )
			{
			case 0:
				Hell_SetGameflag( 2, 0 );	//タイムエクステンドモード OFF
				break;
			case 1:
				Hell_SetGameflag( 2, 1 );	//タイムエクステンドモード ON
				break;
			case 2:
				Hell_SetGameflag( 2, 2 );	//アーケードモード
				break;
			default:
				break;
			}
			setMsg(GameState.MSG_REQ_GAME);
		}

		if(keypad.isPushLeft())
		{
			switch ( select )
			{
			case 0:
				star--;
				if ( star < 0 )
				{
					star = 0;
				}
				break;
			case 1:
				mode--;
				if ( mode < 0 )
				{
					mode = 0;
				}
				break;
			default:
				break;
			}
		}
		else if(keypad.isPushRight())
		{
			switch ( select )
			{
			case 0:
				star++;
				if ( star > maxstar )
				{
					star = maxstar;
				}
				break;
			case 1:
				mode++;
				if ( mode > 2 )
				{
					mode = 2;
				}
				break;
			default:
				break;
			}
		}
	}
	
	void draw()
	{
		Hell_begin();
		resetPerspective();
		Hell_setAlpha(HELL_ALPHA_NORMAL);
	
		Hell_drawTexture("title"
			, 0
			, 0
			, 0
			, 0
			, 640 , 480
			, 1.0 , 1.0
			, 0 );

		Hell_drawFont( "HI SCORE" , 400, 10 , 2 , 255,255,255);
		Hell_drawFont( std.string.toString( hi_score[mode] ) , 400, 30 , 4 , 255,255,255);

		Hell_drawFont( "PRESS Z OR 1 BUTTON TO START"  , 100, 300 , 2 , 255,255,255);
		Hell_drawFont( "PRESS X OR 2 BUTTON TO REPLAY" , 100, 320 , 2 , 255,255,255);
		Hell_drawFont( ">"  , 40, 240 + ( select * 20 ), 2 , 255,255,255);


		char[] str;
		if ( star == 0 )
		{
			str = std.string.format( "LR KEY TO STAR SELECT : RANDOM" );
		}
		else 
		{
			str = std.string.format( "LR KEY TO STAR SELECT : %02d", star );
		}
		Hell_drawFont( str  , 60, 240 , 2 , 255,255,255);

		switch ( mode )
		{
		case 0:
			str = std.string.format( "LR KEY TO MODE SELECT : STANDARD" );
			break;
		case 1:
			str = std.string.format( "LR KEY TO MODE SELECT : ITEM EXTEND" );
			break;
		case 2:
			str = std.string.format( "LR KEY TO MODE SELECT : ENEMY EXTEND" );
			break;
		default:
			break;
		}
		Hell_drawFont( str  , 60, 260 , 2 , 255,255,255);
				
//		Hell_drawFont("HOGE" , 240, 160 , 4 , 255,255,255);
		Hell_drawFont("ALPHA SECRET BASE 2009" , 280, 480-20 , 2 , 255,255,255);
	}
}

/* TitleMain ********************************************************************/
/*	ロゴの状態																	*/
/********************************************************************************/

class LogoMain : GameState
{
public:
	int timer;
	Keypad keypad;					// キーボード/ゲームパッド取得用
	int init_flag;
	
	this()
	{
		setMsg( GameState.MSG_NONE );
		timer = 0;
		init_flag = 0;
		
		keypad = new Keypad();
//		menu = new ClickablePool();
//		menu.add("START" , new Vec3(280 , 400 , 0) , new Vec3(96 , 32 , 0));
		
	}
	
	void move()
	{
		if ( init_flag == 0 )
		{
			init_flag = 1;
			Hell_loadTexture("logo","resource/image/logo.bmp" , [0,255,0]);
		}
		timer++;
		
		if ( ( keypad.isPushEnter() ) || ( keypad.isPushCancel() ) || ( timer > 180 ) )
		{
//			Hell_playBgm( "resource/sound/99.ogg" );
			setMsg(GameState.MSG_REQ_TITLE);
			//１ボタン
		}
	}
	
	void draw()
	{
		Hell_begin();
		resetPerspective();
		Hell_setAlpha(HELL_ALPHA_NORMAL);
		
		Hell_drawTexture("logo"
			, 160
			, 120
			, 0
			, 0
			, 320 , 320
			, 1.0 , 1.0
			, 0 );
				
	}
}

